@@ -1,4 +1,4 @@
 # http://inamidst.com/saxo/
 # Created by Sean B. Palmer
 
-from .saxo import *
+from .core import *
