@@ -242,7 +242,11 @@ def process(env, cmd, path, arg):
         # Might have been removed just after running this thread
         return
     else:
-        try: outs, errs = proc.communicate(octets + b"\n", timeout=12)
+        authorised = "SAXO_AUTHORISED" in env
+        private = not env.get("SAXO_SENDER", "#").startswith("#")
+        timeout = 36 if (authorised and private) else 12
+
+        try: outs, errs = proc.communicate(octets + b"\n", timeout=timeout)
         except subprocess.TimeoutExpired:
             proc.kill()
             # TODO: Use actual prefix
@@ -313,20 +317,48 @@ class Saxo(object):
         common.populate(saxo_path, self.base)
 
         # Load events
+        first = not self.events
         self.events.clear()
+
+        def module_exists(name):
+            try: imp.find_module(name)
+            except ImportError:
+                return False
+            else: return True
+
+        if first and module_exists("plugins"):
+            debug("Warning: a 'plugins' module already exists")
+
+        if first and ("plugins" in sys.modules):
+            raise ImportError("'plugins' duplicated")
+
+        # This means we're using plugins as a namespace module
+        # Might have to move saxo.path's plugins/ to something else
+        # Otherwise it gets unionised into the namespace module
+        # if self.base not in sys.path:
+        # - not needed, because we clear up below
+        sys.path[:0] = [self.base]
+
         plugins = os.path.join(self.base, "plugins")
-        sys.path[:0] = [plugins]
+        plugins_package = importlib.__import__("plugins")
+        if next(iter(plugins_package.__path__)) != plugins:
+            # This is very unlikely to happen, because we pushed self.base
+            # to the front of sys.path, but perhaps some site configuration
+            # or other import mechanism may affect this
+            raise ImportError("non-saxo 'plugins' module")
 
         setups = {}
         for name in os.listdir(plugins):
             if ("_" in name) or (not name.endswith(".py")):
                 continue
 
-            name = name[:-3]
+            name = "plugins." + name[:-3]
             if not name in sys.modules:
                 try: module = importlib.import_module(name)
                 except Exception as err:
                     debug("Error loading %s:" % name, err)
+            elif first:
+                raise ImportError("%r duplicated" % name)
             else:
                 module = sys.modules[name]
                 try: module = imp.reload(module)
@@ -351,7 +383,8 @@ class Saxo(object):
 
         graph = {}
         for setup in setups.values():
-            graph[setup.saxo_name] = setup.saxo_deps
+            deps = ["plugins." + dep for dep in setup.saxo_deps]
+            graph[setup.saxo_name] = deps
 
         database_filename = os.path.join(self.base, "database.sqlite3")
         with sqlite.Database(database_filename) as self.db:
