@@ -6,6 +6,7 @@ import codecs
 import os
 import signal
 import sys
+import threading
 import time
 
 # Save PEP 3122!
@@ -47,6 +48,16 @@ def action(function):
     return function
 action.names = {}
 
+lock = threading.Lock()
+
+def debug(*args, **kargs):
+    with lock:
+        try:
+            print(*args, **kargs)
+            sys.stdout.flush()
+        except BrokenPipeError:
+            sys.exit()
+
 # Options
 
 def version(args, v):
@@ -234,26 +245,27 @@ def log(args):
 
 @action
 def start(args):
-    base = base_option(args)
+    import json
 
     # Quit any previously connected instances
     if "." in __name__:
-        from .saxo import client
+        from .saxo import client, data
     else:
-        from saxo import client
+        from saxo import client, data
 
+    base = base_option(args)
     try: client("quit", base=base)
     except FileNotFoundError as err:
         ...
     except ConnectionRefusedError as err:
         ...
     except PermissionError as err:
-        print("Error: Unable to connect to internal socket", file=sys.stderr)
-        print("Check permissions on the config dir", file=sys.stderr)
+        debug("Error: Unable to connect to internal socket", file=sys.stderr)
+        debug("Check permissions on the config dir", file=sys.stderr)
         sys.stderr.flush()
         sys.exit()
     else:
-        print("Warning: Client may already have been running!")
+        debug("Warning: Client may already have been running!")
         sys.stdout.flush()
 
     pidfile = os.path.join(base, "pid")
@@ -287,6 +299,12 @@ def start(args):
                 os.remove(pidfile)
         atexit.register(delete_pidfile)
 
+    if args.action == "start":
+        # Otherwise you get recursion if args.action == "restart"
+        os.environ["SAXO_BASE"] = base
+        sys_args = json.dumps(args.original)
+        data("args", sys_args, command="script.py", check=False)
+    
     # Save PEP 3122!
     if "." in __name__:
         from . import irc
@@ -296,6 +314,30 @@ def start(args):
     irc.start(base)
     return 0
 
+@action
+def restart(args):
+    import json
+
+    base = base_option(args)
+    os.environ["SAXO_BASE"] = base
+
+    if "." in __name__:
+        from .saxo import data, version
+    else:
+        from saxo import data, version
+
+    sys_args = json.loads(data("args", command="script.py", check=False))
+    if sys_args[0] == "restart":
+        debug("Recursion detected!", sys_args)
+        debug("Not restarting")
+        sys.exit(1)
+
+    debug("Stopping existing instance...")
+    stop(args)
+    argv = [sys.argv[0]] + sys_args
+    debug("Restarting using:", argv)
+    main(argv, version)
+
 @action
 def status(args):
     base = base_option(args)
@@ -308,11 +350,11 @@ def status(args):
 
     try: client("noop", base=base)
     except FileNotFoundError as err:
-        print("not running")
+        debug("not running")
     except ConnectionRefusedError as err:
-        print("not running")
+        debug("not running")
     else:
-        print("running")
+        debug("running")
 
 @action
 def stop(args):
@@ -499,6 +541,7 @@ def main(argv, v):
         help="the path to the saxo configuration directory")
 
     args = parser.parse_args(argv[1:])
+    args.original = argv[1:]
 
     if args.output:
         print("Sorry, -o / --output has been removed!")
@@ -516,6 +559,21 @@ def main(argv, v):
             code = action.names[args.action](args)
             if isinstance(code, int):
                 sys.exit(code)
+        elif args.action.startswith("."):
+            # Save PEP 3122!
+            if "." in __name__:
+                from . import saxo
+            else:
+                import saxo
+
+            def do(cmd, arg):
+                import subprocess
+                cmd = os.path.join(saxo.path, "commands", cmd)
+                try: octets = subprocess.check_output([cmd, arg])
+                except Exception as err:
+                    raise err # return "Error: %s" % err
+                return octets.decode("utf-8").rstrip("\r\n")
+            print(do(args.action[1:], args.directory or ""))
         else:
             common.error("unrecognised action: %s" % args.action, code=2)
 
