@@ -13,6 +13,10 @@ _regex_key = re.compile(r'([^=]+)')
 _regex_value = re.compile(r'("[^"\\]*(?:\\.[^"\\]*)*"|[^;]+)')
 
 user_agent = "Mozilla/5.0 (Services)"
+modern_user_agent = " ".join([
+    "Mozilla/5.0",
+    "(Macintosh; Intel Mac OS X 10.9; rv:26.0)"
+    "Gecko/20100101 Firefox/26.0"])
 
 def content_type(info):
     mime = None
@@ -70,15 +74,19 @@ def decode_entities(hypertext):
             return chr(html.entities.name2codepoint[name])
         return "[" + name + "]"
 
-    return _regex_entity.sub(entity, hypertext)
+    def default(match):
+        try: return entity(match)
+        except: return match.group(1)
+    return _regex_entity.sub(default, hypertext)
 
 def request(url, query=None, data=None, method="GET",
-        limit=None, follow=True, headers=None):
+        limit=None, follow=True, headers=None, modern=False):
     headers = {} if (headers is None) else headers
 
     response = {}
     if "User-Agent" not in headers:
-        headers["User-Agent"] = user_agent
+        modern = modern is True
+        headers["User-Agent"] = modern_user_agent if modern else user_agent
     response["request-headers"] = headers
 
     safe = "".join(chr(i) for i in range(0x01, 0x80))
@@ -164,16 +172,17 @@ def request(url, query=None, data=None, method="GET",
 
     if "octets" in response:
         def guess_encoding(response):
-            try: response["text"] = response["octets"].decode("iso-8859-1")
+            try: response["text"] = response["octets"].decode("utf-8")
             except UnicodeDecodeError:
-                response["text"] = response["octets"].decode("utf-8", "replace")
-                response["encoding"] = "utf-8"
-            else:
+                response["text"] = response["octets"].decode("iso-8859-1")
                 response["encoding"] = "iso-8859-1"
+            else:
+                response["encoding"] = "utf-8"
             response["encoding-source"] = "heuristic"
 
         if "encoding" in response:
-            try: response["text"] = response["octets"].decode(response["encoding"])
+            encoding = response["encoding"]
+            try: response["text"] = response["octets"].decode(encoding)
             except (UnicodeDecodeError, LookupError):
                 guess_encoding(response)
         else:
